@@ -29,30 +29,33 @@ The base class in PluginBase will serve as documentation of available.
 
 import inspect
 import os
-import shutil
 from optparse import OptionConflictError, OptionGroup
 
 import nuitka.plugins.commercial
 import nuitka.plugins.standard
 from nuitka import Options, OutputDirectories
-from nuitka.__past__ import basestring, iter_modules
+from nuitka.__past__ import basestring, iter_modules, iterItems
 from nuitka.build.DataComposerInterface import deriveModuleConstantsBlobName
 from nuitka.containers.odict import OrderedDict
 from nuitka.containers.oset import OrderedSet
-from nuitka.Errors import NuitkaPluginError
 from nuitka.freezer.IncludedEntryPoints import makeDllEntryPointOld
 from nuitka.ModuleRegistry import addUsedModule
 from nuitka.Tracing import plugins_logger, printLine
 from nuitka.utils.FileOperations import (
     addFileExecutablePermission,
+    copyFile,
     makePath,
     putTextFileContents,
-    relpath,
 )
 from nuitka.utils.Importing import importFileAsModule
-from nuitka.utils.ModuleNames import ModuleName
+from nuitka.utils.ModuleNames import ModuleName, checkModuleName
 
-from .PluginBase import NuitkaPluginBase, post_modules, pre_modules
+from .PluginBase import (
+    NuitkaPluginBase,
+    makeTriggerModuleName,
+    postload_trigger_name,
+    preload_trigger_name,
+)
 
 # Maps plugin name to plugin instances.
 active_plugins = OrderedDict()
@@ -62,6 +65,11 @@ plugin_datatag2pluginclasses = {}
 plugin_values = {}
 user_plugins = OrderedSet()
 
+# Trigger modules
+pre_modules = {}
+post_modules = {}
+fake_modules = {}
+
 
 def _addActivePlugin(plugin_class, args, force=False):
     plugin_name = plugin_class.plugin_name
@@ -78,7 +86,11 @@ def _addActivePlugin(plugin_class, args, force=False):
     else:
         plugin_args = {}
 
-    plugin_instance = plugin_class(**plugin_args)
+    try:
+        plugin_instance = plugin_class(**plugin_args)
+    except TypeError as e:
+        plugin_class.sysexit("Problem initializing plugin: %s" % e)
+
     assert isinstance(plugin_instance, NuitkaPluginBase), plugin_instance
 
     active_plugins[plugin_name] = plugin_instance
@@ -258,26 +270,34 @@ class Plugins(object):
 
         result = []
 
-        for full_name in plugin.getImplicitImports(module):
-            if type(full_name) in (tuple, list):
-                raise NuitkaPluginError(
-                    "Plugin %r needs to be change to only return modules names, not %r"
-                    % (plugin, full_name)
-                )
+        def iterateModuleNames(value):
+            for v in value:
+                if type(v) in (tuple, list):
+                    plugin.sysexit(
+                        "Plugin %r needs to be change to only return modules names, not %r (for %s)"
+                        % (plugin.plugin_name, v, module.getFullName().asString())
+                    )
 
-            full_name = ModuleName(full_name)
+                if inspect.isgenerator(v):
+                    for w in iterateModuleNames(v):
+                        yield w
 
+                    return
+
+                if not checkModuleName(v):
+                    plugin.sysexit(
+                        "Plugin %r returned an invalid module name, not %r (for %s)"
+                        % (plugin, v, module.getFullName().asString())
+                    )
+
+                yield ModuleName(v)
+
+        for full_name in iterateModuleNames(plugin.getImplicitImports(module)):
             try:
-                _module_package, module_filename, _finding = Importing.findModule(
-                    importing=module,
+                _module_name, module_filename, _finding = Importing.locateModule(
                     module_name=full_name,
                     parent_package=None,
-                    level=-1,
-                    warn=False,
-                )
-
-                module_filename = plugin.locateModule(
-                    importing=module, module_name=full_name
+                    level=0,
                 )
             except Exception:
                 plugin.warning(
@@ -306,7 +326,7 @@ class Plugins(object):
         return result
 
     @staticmethod
-    def _reportImplicitImports(implicit_imports, signal_change):
+    def _reportImplicitImports(plugin, module, implicit_imports, signal_change):
         from nuitka.importing import Recursion
         from nuitka.importing.Importing import getModuleNameAndKindFromFilename
 
@@ -325,14 +345,19 @@ class Plugins(object):
             if decision:
                 imported_module = Recursion.recurseTo(
                     signal_change=signal_change,
-                    module_package=full_name.getPackageName(),
+                    module_name=full_name,
                     module_filename=module_filename,
-                    module_relpath=relpath(module_filename),
                     module_kind=module_kind,
                     reason=reason,
                 )
 
-                addUsedModule(imported_module)
+                addUsedModule(
+                    module=imported_module,
+                    using_module=module,
+                    usage_tag="plugin:" + plugin.plugin_name,
+                    reason=reason,
+                    source_ref=module.source_ref,
+                )
 
     @classmethod
     def considerImplicitImports(cls, module, signal_change):
@@ -345,6 +370,8 @@ class Plugins(object):
                 )
 
             cls._reportImplicitImports(
+                plugin=plugin,
+                module=module,
                 implicit_imports=cls.implicit_imports_cache[key],
                 signal_change=signal_change,
             )
@@ -353,10 +380,41 @@ class Plugins(object):
         full_name = module.getFullName()
 
         if full_name in pre_modules:
-            addUsedModule(pre_modules[full_name])
+            addUsedModule(
+                pre_modules[full_name],
+                using_module=module,
+                usage_tag="plugins",
+                reason="Not yet propagated by plugins.",
+                source_ref=module.source_ref,
+            )
 
         if full_name in post_modules:
-            addUsedModule(post_modules[full_name])
+            addUsedModule(
+                module=post_modules[full_name],
+                using_module=module,
+                usage_tag="plugins",
+                reason="Not yet propagated by plugins.",
+                source_ref=module.source_ref,
+            )
+
+        if full_name in fake_modules:
+            for fake_module, plugin, reason in fake_modules[full_name]:
+                addUsedModule(
+                    module=fake_module,
+                    using_module=module,
+                    usage_tag="plugins",
+                    reason=reason,
+                    source_ref=module.source_ref,
+                )
+
+    @staticmethod
+    def onCopiedDLLs(dist_dir, used_dlls):
+        """Lets the plugins modify copied DLLs on disk."""
+        for dll_filename, _sources in iterItems(used_dlls):
+            for plugin in getActivePlugins():
+                plugin.onCopiedDLL(
+                    os.path.join(dist_dir, os.path.basename(dll_filename))
+                )
 
     @staticmethod
     def onStandaloneDistributionFinished(dist_dir):
@@ -370,6 +428,12 @@ class Plugins(object):
         for plugin in getActivePlugins():
             plugin.onStandaloneDistributionFinished(filename)
 
+    @staticmethod
+    def onBootstrapBinary(filename):
+        """Let plugins add to bootstrap binary in some way"""
+        for plugin in getActivePlugins():
+            plugin.onBootstrapBinary(filename)
+
     @staticmethod
     def onFinalResult(filename):
         """Let plugins add to final binary in some way"""
@@ -414,7 +478,7 @@ class Plugins(object):
 
                     makePath(os.path.dirname(extra_dll.dest_path))
 
-                    shutil.copyfile(extra_dll.source_path, extra_dll.dest_path)
+                    copyFile(extra_dll.source_path, extra_dll.dest_path)
 
                     if extra_dll.executable:
                         addFileExecutablePermission(extra_dll.dest_path)
@@ -494,7 +558,7 @@ class Plugins(object):
 
         Args:
             module: the module object (serves as dict key)
-            trigger_name: string ("-preload"/"-postload")
+            trigger_name: string ("preload"/"postload")
             code: the code string
 
         Returns
@@ -503,7 +567,7 @@ class Plugins(object):
 
         from nuitka.tree.Building import buildModule
 
-        module_name = ModuleName(module.getFullName() + trigger_name)
+        module_name = makeTriggerModuleName(module.getFullName(), trigger_name)
 
         # In debug mode, put the files in the build folder, so they can be looked up easily.
         if Options.is_debug and "HIDE_SOURCE" not in flags:
@@ -519,11 +583,11 @@ class Plugins(object):
                     os.path.dirname(module.getCompileTimeFilename()),
                     module_name.asPath() + ".py",
                 ),
-                module_package=module_name.getPackageName(),
+                module_name=module_name,
                 source_code=code,
                 is_top=False,
                 is_main=False,
-                is_shlib=False,
+                is_extension=False,
                 is_fake=module_name,
                 hide_syntax_error=False,
             )
@@ -539,6 +603,9 @@ class Plugins(object):
 
     @classmethod
     def onModuleDiscovered(cls, module):
+        # We offer plugins many ways to provide extra stuff
+        # pylint: disable=too-many-branches,too-many-locals,too-many-statements
+
         full_name = module.getFullName()
 
         def _untangleLoadDesc(descs):
@@ -560,8 +627,21 @@ class Plugins(object):
 
                     yield plugin, code, reason, flags
 
+        def _untangleFakeDesc(descs):
+            if descs and inspect.isgenerator(descs):
+                descs = tuple(descs)
+
+            if descs:
+                if type(descs[0]) not in (tuple, list):
+                    descs = [descs]
+
+                for desc in descs:
+                    assert len(desc) == 4, desc
+                    yield plugin, desc[0], desc[1], desc[2], desc[3]
+
         preload_descs = []
         postload_descs = []
+        fake_descs = []
 
         for plugin in getActivePlugins():
             plugin.onModuleDiscovered(module)
@@ -572,6 +652,9 @@ class Plugins(object):
             postload_descs.extend(
                 _untangleLoadDesc(descs=plugin.createPostModuleLoadCode(module))
             )
+            fake_descs.extend(
+                _untangleFakeDesc(descs=plugin.createFakeModuleDependency(module))
+            )
 
         if preload_descs:
             total_code = []
@@ -593,7 +676,7 @@ class Plugins(object):
 
                 pre_modules[full_name] = cls._createTriggerLoadedModule(
                     module=module,
-                    trigger_name="-preLoad",
+                    trigger_name=preload_trigger_name,
                     code="\n\n".join(total_code),
                     flags=total_flags,
                 )
@@ -618,11 +701,39 @@ class Plugins(object):
 
                 post_modules[full_name] = cls._createTriggerLoadedModule(
                     module=module,
-                    trigger_name="-postLoad",
+                    trigger_name=postload_trigger_name,
                     code="\n\n".join(total_code),
                     flags=total_flags,
                 )
 
+        if fake_descs:
+            fake_modules[full_name] = []
+
+            from nuitka.tree.Building import buildModule
+
+            for (
+                plugin,
+                fake_module_name,
+                source_code,
+                fake_filename,
+                reason,
+            ) in fake_descs:
+                fake_module, _added = buildModule(
+                    module_filename=fake_filename,
+                    module_name=fake_module_name,
+                    source_code=source_code,
+                    is_top=False,
+                    is_main=False,
+                    is_extension=False,
+                    is_fake=fake_module_name,
+                    hide_syntax_error=False,
+                )
+
+                if fake_module.getCompilationMode() == "bytecode":
+                    fake_module.setSourceCode(source_code)
+
+                fake_modules[full_name].append((fake_module, plugin, reason))
+
     @staticmethod
     def onModuleSourceCode(module_name, source_code):
         assert type(module_name) is ModuleName
@@ -708,44 +819,28 @@ class Plugins(object):
             plugin.onModuleCompleteSet(module_set)
 
     @staticmethod
-    def considerFailedImportReferrals(module_name):
-        for plugin in getActivePlugins():
-            new_module_name = plugin.considerFailedImportReferrals(module_name)
-
-            if new_module_name is not None:
-                return ModuleName(new_module_name)
-
-        return None
-
-    @staticmethod
-    def suppressUnknownImportWarning(importing, module_name):
+    def suppressUnknownImportWarning(importing, source_ref, module_name):
         """Let plugins decide whether to suppress import warnings for an unknown module.
 
         Notes:
             If all plugins return False or None, the return will be False, else True.
         Args:
             importing: the module which is importing "module_name"
+            source_ref: pointer to file source code or bytecode
             module_name: the module to be imported
         returns:
             True or False (default)
         """
-        if importing.isCompiledPythonModule() or importing.isPythonShlibModule():
-            importing_module = importing
-        else:
-            importing_module = importing.getParentModule()
-
         source_ref = importing.getSourceReference()
 
         for plugin in getActivePlugins():
-            if plugin.suppressUnknownImportWarning(
-                importing_module, module_name, source_ref
-            ):
+            if plugin.suppressUnknownImportWarning(importing, module_name, source_ref):
                 return True
 
         return False
 
     @staticmethod
-    def decideCompilation(module_name, source_ref):
+    def decideCompilation(module_name):
         """Let plugins decide whether to C compile a module or include as bytecode.
 
         Notes:
@@ -755,7 +850,7 @@ class Plugins(object):
             "compiled" (default) or "bytecode".
         """
         for plugin in getActivePlugins():
-            value = plugin.decideCompilation(module_name, source_ref)
+            value = plugin.decideCompilation(module_name)
 
             if value is not None:
                 assert value in ("compiled", "bytecode")
@@ -788,13 +883,37 @@ class Plugins(object):
                     assert type(value) is dict, value
 
                     # We order per plugin, but from the plugins, lets just take a dict
-                    # and achieve determism by ordering the defines by name.
+                    # and achieve determinism by ordering the defines by name.
                     for key, value in sorted(value.items()):
                         # False alarm, pylint: disable=I0021,unsupported-assignment-operation
                         cls.preprocessor_symbols[key] = value
 
         return cls.preprocessor_symbols
 
+    extra_include_directories = None
+
+    @classmethod
+    def getExtraIncludeDirectories(cls):
+        """Let plugins extra directories to use for C includes in compilation.
+
+        Notes:
+            The plugins can each contribute, but are hopefully not colliding,
+            order will be plugin order.
+
+        Returns:
+            OrderedSet() of paths to include as well.
+        """
+        if cls.extra_include_directories is None:
+            cls.extra_include_directories = OrderedSet()
+
+            for plugin in getActivePlugins():
+                value = plugin.getExtraIncludeDirectories()
+
+                if value:
+                    cls.extra_include_directories.update(value)
+
+        return cls.extra_include_directories
+
     @staticmethod
     def getExtraCodeFiles():
         result = OrderedDict()
@@ -806,7 +925,7 @@ class Plugins(object):
                 assert type(value) is dict
 
                 # We order per plugin, but from the plugins, lets just take a dict
-                # and achieve determism by ordering the files by name.
+                # and achieve determinism by ordering the files by name.
                 for key, value in sorted(value.items()):
                     if not key.startswith("nuitka_"):
                         key = "plugin." + plugin.plugin_name + "." + key
@@ -989,6 +1108,20 @@ def loadPlugins():
         loadStandardPluginClasses()
 
 
+def addStandardPluginCommandlineOptions(parser, data_files_tags):
+    loadPlugins()
+
+    for (_plugin_name, (plugin_class, _plugin_detector)) in sorted(
+        plugin_name2plugin_classes.items()
+    ):
+        if plugin_class.isAlwaysEnabled():
+            _addPluginCommandLineOptions(
+                parser=parser,
+                plugin_class=plugin_class,
+                data_files_tags=data_files_tags,
+            )
+
+
 def activatePlugins():
     """Activate selected plugin classes
 
@@ -1172,3 +1305,30 @@ def getPluginOptions(plugin_name):
         result[option.dest] = arg_value
 
     return result
+
+
+def replaceTriggerModule(old, new):
+    """Replace a trigger module with another form if it. For use in bytecode demotion."""
+
+    found = None
+    for key, value in pre_modules.items():
+        if value is old:
+            found = key
+            break
+
+    if found is not None:
+        pre_modules[found] = new
+
+    found = None
+    for key, value in post_modules.items():
+        if value is old:
+            found = key
+            break
+
+    if found is not None:
+        post_modules[found] = new
+
+
+def isTriggerModule(module):
+    """Decide of a module is a trigger module."""
+    return module in pre_modules.values() or module in post_modules.values()
