@@ -23,25 +23,36 @@ to add to this and submit patches to make it more complete.
 """
 
 import fnmatch
-import os
-import sys
 
 from nuitka.__past__ import iter_modules
 from nuitka.containers.oset import OrderedSet
+from nuitka.Options import (
+    isOnefileMode,
+    isStandaloneMode,
+    mayDisableConsoleWindow,
+    shallCreateAppBundle,
+    shallDisableConsoleWindow,
+)
 from nuitka.plugins.PluginBase import NuitkaPluginBase
 from nuitka.PythonVersions import python_version
 from nuitka.utils.ModuleNames import ModuleName
-from nuitka.utils.SharedLibraries import getPyWin32Dir
-from nuitka.utils.Utils import getOS, isLinux, isMacOS, isWin32Windows
+from nuitka.utils.Utils import getOS, isMacOS, isWin32Windows
 from nuitka.utils.Yaml import parsePackageYaml
 
 
-class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
+class NuitkaPluginImplicitImports(NuitkaPluginBase):
     plugin_name = "implicit-imports"
 
     def __init__(self):
         self.config = parsePackageYaml(__package__, "implicit-imports.yml")
 
+        for key in self.config.keys():
+            if "/" in key:
+                self.sysexit(
+                    "Error, invalid key in 'implicit-imports.yml' looks like a file path, not module name '%s'."
+                    % key
+                )
+
     @staticmethod
     def isAlwaysEnabled():
         return True
@@ -104,9 +115,75 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
 
         # Checking for config, but also allowing fall through.
         if config:
-            dependencies = config.get("depends")
+            dependencies = config.get("depends", [])
+            standalone_macos_bundle_mode = config.get("standalone_macos_bundle_mode")
+            disable_console = config.get("disable_console")
 
-            if type(dependencies) is not list or not dependencies:
+            if standalone_macos_bundle_mode is not None:
+                if standalone_macos_bundle_mode is True:
+                    standalone_macos_bundle_mode = "yes"
+                if standalone_macos_bundle_mode is False:
+                    standalone_macos_bundle_mode = "no"
+
+                if type(standalone_macos_bundle_mode) is not str:
+                    self.sysexit(
+                        "Error, requiring str as 'standalone_macos_bundle_mode' value for '%s' entry."
+                        % full_name
+                    )
+
+                if standalone_macos_bundle_mode not in ("yes", "no", "onefile"):
+                    self.sysexit(
+                        "Error, illegal value for 'standalone_macos_bundle_mode' for '%s' entry."
+                        % full_name
+                    )
+
+                if isMacOS():
+                    if standalone_macos_bundle_mode == "yes":
+                        if isStandaloneMode() and not shallCreateAppBundle():
+                            self.sysexit(
+                                """\
+Error, package '%s' requires '--macos-create-app-bundle' to be used or else it cannot work."""
+                                % full_name
+                            )
+                    elif standalone_macos_bundle_mode == "onefile":
+                        if shallCreateAppBundle() and not isOnefileMode():
+                            self.sysexit(
+                                """\
+Error, package '%s' requires '--onefile' to be used on top of '--macos-create-app-bundle' or else it cannot work."""
+                                % full_name
+                            )
+
+            if disable_console is not None:
+                if disable_console is True:
+                    disable_console = "yes"
+                if disable_console is False:
+                    disable_console = "no"
+
+                if type(disable_console) is not str:
+                    self.sysexit(
+                        "Error, requiring str as 'disable_console' value for '%s' entry."
+                        % full_name
+                    )
+
+                # Required on macOS is recommended elsewhere.
+                if not isMacOS() and disable_console == "macos":
+                    disable_console = "recommend"
+
+                if (
+                    disable_console == "recommend"
+                    and mayDisableConsoleWindow()
+                    and not shallDisableConsoleWindow()
+                ):
+                    self.info(
+                        "Note, when using '%s', consider using '--disable-console' option."
+                    )
+
+            if (
+                type(dependencies) is not list
+                and not dependencies
+                and standalone_macos_bundle_mode is None
+                and disable_console is None
+            ):
                 self.sysexit(
                     "Error, requiring list below 'depends' entry for '%s' entry."
                     % full_name
@@ -1177,7 +1254,7 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
     def getImplicitImports(self, module):
         full_name = module.getFullName()
 
-        if module.isPythonShlibModule():
+        if module.isPythonExtensionModule():
             for used_module in module.getUsedModules():
                 yield used_module[0]
 
@@ -1194,7 +1271,7 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
                 "importlib_resources",
                 "more_itertools",
             ):
-                yield "pkg_resources._vendor." + part.strip("' ")
+                yield "pkg_resources._vendor." + part
 
         else:
             # create a flattened import set for full_name and yield from it
@@ -1216,82 +1293,6 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
         # Do nothing by default.
         return source_code
 
-    def getExtraDlls(self, module):
-        full_name = module.getFullName()
-
-        if full_name == "uuid" and isLinux():
-            uuid_dll_path = self.locateDLL("uuid")
-
-            if uuid_dll_path is not None:
-                yield self.makeDllEntryPoint(
-                    uuid_dll_path, os.path.basename(uuid_dll_path), None
-                )
-        elif full_name == "iptc" and isLinux():
-            import iptc.util  # pylint: disable=I0021,import-error
-
-            xtwrapper_dll = iptc.util.find_library("xtwrapper")[0]
-            xtwrapper_dll_path = xtwrapper_dll._name  # pylint: disable=protected-access
-
-            yield self.makeDllEntryPoint(
-                xtwrapper_dll_path, os.path.basename(xtwrapper_dll_path), None
-            )
-        elif full_name == "coincurve._libsecp256k1" and isWin32Windows():
-            yield self.makeDllEntryPoint(
-                os.path.join(module.getCompileTimeDirectory(), "libsecp256k1.dll"),
-                os.path.join(full_name.getPackageName(), "libsecp256k1.dll"),
-                full_name.getPackageName(),
-            )
-        # TODO: This should be its own plugin.
-        elif (
-            full_name
-            in (
-                "pythoncom",
-                "win32api",
-                "win32clipboard",
-                "win32console",
-                "win32cred",
-                "win32crypt",
-                "win32event",
-                "win32evtlog",
-                "win32file",
-                "win32gui",
-                "win32help",
-                "win32inet",
-                "win32job",
-                "win32lz",
-                "win32net",
-                "win32pdh",
-                "win32pipe",
-                "win32print",
-                "win32process",
-                "win32profile",
-                "win32ras",
-                "win32security",
-                "win32service",
-                "win32trace",
-                "win32transaction",
-                "win32ts",
-                "win32wnet",
-            )
-            and isWin32Windows()
-        ):
-            pywin_dir = getPyWin32Dir()
-
-            if pywin_dir is not None:
-                for dll_name in "pythoncom", "pywintypes":
-
-                    pythoncom_filename = "%s%d%d.dll" % (
-                        dll_name,
-                        sys.version_info[0],
-                        sys.version_info[1],
-                    )
-                    pythoncom_dll_path = os.path.join(pywin_dir, pythoncom_filename)
-
-                    if os.path.exists(pythoncom_dll_path):
-                        yield self.makeDllEntryPoint(
-                            pythoncom_dll_path, pythoncom_filename, None
-                        )
-
     unworthy_namespaces = (
         "setuptools",  # Not performance relevant.
         "distutils",  # Not performance relevant.
@@ -1319,8 +1320,11 @@ class NuitkaPluginPopularImplicitImports(NuitkaPluginBase):
         "telethon.tl.types",  # Not performance relevant and slow C compile
         "importlib_metadata",  # Not performance relevant and slow C compile
         "comtypes.gen",  # Not performance relevant and slow C compile
+        "win32com.gen_py",  # Not performance relevant and slow C compile
         "phonenumbers.geodata",  # Not performance relevant and slow C compile
         "site",  # Not performance relevant and problems with .pth files
+        "packaging",  # Not performance relevant.
+        "appdirs",  # Not performance relevant.
     )
 
     def decideCompilation(self, module_name):
