@@ -18,12 +18,12 @@
 """ Details see below in class definition.
 """
 import os
-import shutil
 import sys
 
 from nuitka import Options
+from nuitka.freezer.IncludedDataFiles import makeIncludedDataDirectory
 from nuitka.plugins.PluginBase import NuitkaPluginBase
-from nuitka.utils.FileOperations import copyTree
+from nuitka.utils.Utils import isWin32Windows
 
 
 def _isTkInterModule(module):
@@ -31,29 +31,26 @@ def _isTkInterModule(module):
     return full_name in ("Tkinter", "tkinter", "PySimpleGUI", "PySimpleGUI27")
 
 
-class TkinterPlugin(NuitkaPluginBase):
-    """This class represents the main logic of the plugin.
+class NuitkaPluginTkinter(NuitkaPluginBase):
+    """This class represents the main logic of the TkInter plugin.
 
-    This is a plug-in to make programs work well in standalone mode which are using tkinter.
-    These programs require the presence of certain libraries written in the TCL language.
-    On Windows platforms, and even on Linux, the existence of these libraries cannot be
-    assumed. We therefore
+     This is a plug-in to make programs work well in standalone mode which are using tkinter.
+     These programs require the presence of certain libraries written in the TCL language.
+     On Windows platforms, and even on Linux, the existence of these libraries cannot be
+     assumed. We therefore
 
-    1. Copy the TCL libraries as sub-folders to the program's dist folder
-    2. Redirect the program's tkinter requests to these library copies. This is
-       done by setting appropriate variables in the os.environ dictionary.
-       Tkinter will use these variable value to locate the library locations.
+     1. Copy the TCL libraries as sub-folders to the program's dist folder
+     2. Redirect the program's tkinter requests to these library copies. This is
+        done by setting appropriate variables in the os.environ dictionary.
+        Tkinter will use these variable value to locate the library locations.
 
-    Each time before the program issues an import to a tkinter module, we make
-    sure, that the TCL environment variables are correctly set.
-
-    Args:
-        NuitkaPluginBase: the plugin template class we are inheriting.
+     Each time before the program issues an import to a tkinter module, we make
+     sure, that the TCL environment variables are correctly set.
 
     Notes:
-        You can enforce using a specific TCL folder by using TCL_LIBRARY
-        and a Tk folder by using TK_LIBRARY, but that ought to normally
-        not be necessary.
+         You can enforce using a specific TCL folder by using TCL_LIBRARY
+         and a Tk folder by using TK_LIBRARY, but that ought to normally
+         not be necessary.
     """
 
     plugin_name = "tk-inter"  # Nuitka knows us by this name
@@ -71,7 +68,7 @@ class TkinterPlugin(NuitkaPluginBase):
         """This method is called one time only to check, whether the plugin might make sense at all.
 
         Returns:
-            True if this is a standalone compilation on Windows, else False.
+            True if this is a standalone, else False.
         """
         return Options.isStandaloneMode()
 
@@ -89,16 +86,15 @@ class TkinterPlugin(NuitkaPluginBase):
             Code to insert and None (tuple)
         """
         # only insert code for tkinter related modules
-        if not _isTkInterModule(module):
-            return None, None
+        if _isTkInterModule(module):
+            # The following code will be executed before importing the module.
+            # If required we set the respective environment values.
+            code = r"""
+import os
+os.environ["TCL_LIBRARY"] = os.path.join(__nuitka_binary_dir, "tcl")
+os.environ["TK_LIBRARY"] = os.path.join(__nuitka_binary_dir, "tk")"""
 
-        # The following code will be executed before importing the module.
-        # If required we set the respective environment values.
-        code = """import os
-if not os.environ.get("TCL_LIBRARY"):
-    os.environ["TCL_LIBRARY"] = os.path.join(__nuitka_binary_dir, "tcl")
-    os.environ["TK_LIBRARY"] = os.path.join(__nuitka_binary_dir, "tk")"""
-        return code, "Need to make sure we set environment variables for TCL."
+            return code, "Need to make sure we set environment variables for TCL."
 
     @classmethod
     def addPluginCommandLineOptions(cls, group):
@@ -109,8 +105,7 @@ if not os.environ.get("TCL_LIBRARY"):
             default=None,
             help="""\
 The Tk library dir. Nuitka is supposed to automatically detect it, but you can
-override it here. Should it fail to discover, please report the bug. Default
-is automatic detection.""",
+override it here. Default is automatic detection.""",
         )
 
         group.add_option(
@@ -122,50 +117,49 @@ is automatic detection.""",
 The Tcl library dir. See comments for Tk library dir.""",
         )
 
-    def considerExtraDlls(self, dist_dir, module):
-        """Copy TCL libraries to the dist folder.
+    def considerDataFiles(self, module):
+        """Provide TCL libraries to the dist folder.
 
         Notes:
-            We will copy the TCL/TK directories to the program's root directory.
-            The general intention is that we return a tuple of file names.
-            We need however two subdirectories inserted, and therefore do the
-            copy ourselves and return an empty tuple.
+            We will provide the copy the TCL/TK directories to the program's root directory,
+            that might be shiftable with some work.
 
         Args:
-            dist_dir: the name of the program's dist folder
-            module: the module object (not used here)
+            module: the module in question, maybe ours
 
-        Returns:
-            None
+        Yields:
+            IncludedDataFile objects.
         """
         if not _isTkInterModule(module):
-            return ()
-        if self.files_copied:
-            return ()
-        self.files_copied = True
+            return
 
         # Check typical locations of the dirs
         candidates_tcl = (
             os.environ.get("TCL_LIBRARY"),
-            os.path.join(sys.prefix, "tcl", "tcl8.5"),
-            os.path.join(sys.prefix, "tcl", "tcl8.6"),
             "/usr/share/tcltk/tcl8.6",
             "/usr/share/tcltk/tcl8.5",
             "/usr/share/tcl8.6",
             "/usr/share/tcl8.5",
             "/usr/lib64/tcl/tcl8.5",
             "/usr/lib64/tcl/tcl8.6",
+            os.path.join(sys.prefix, "tcl", "tcl8.5"),
+            os.path.join(sys.prefix, "tcl", "tcl8.6"),
+            os.path.join(sys.prefix, "lib", "tcl8.5"),
+            os.path.join(sys.prefix, "lib", "tcl8.6"),
         )
+
         candidates_tk = (
             os.environ.get("TK_LIBRARY"),
-            os.path.join(sys.prefix, "tcl", "tk8.5"),
-            os.path.join(sys.prefix, "tcl", "tk8.6"),
             "/usr/share/tcltk/tk8.6",
             "/usr/share/tcltk/tk8.5",
             "/usr/share/tk8.6",
             "/usr/share/tk8.5",
             "/usr/lib64/tcl/tk8.5",
             "/usr/lib64/tcl/tk8.6",
+            os.path.join(sys.prefix, "tcl", "tk8.5"),
+            os.path.join(sys.prefix, "tcl", "tk8.6"),
+            os.path.join(sys.prefix, "lib", "tk8.5"),
+            os.path.join(sys.prefix, "lib", "tk8.6"),
         )
 
         tcl = self.tcl_library_dir
@@ -175,7 +169,9 @@ The Tcl library dir. See comments for Tk library dir.""",
                     break
 
         if tcl is None or not os.path.exists(tcl):
-            self.sysexit("Could not find Tcl. Aborting standalone generation.")
+            self.sysexit(
+                "Could not find Tcl, you might need to set 'TCL_LIBRARY' and if that works, report a bug."
+            )
 
         tk = self.tk_library_dir
         if tk is None:
@@ -183,34 +179,40 @@ The Tcl library dir. See comments for Tk library dir.""",
                 if tk is not None and os.path.exists(tk):
                     break
 
-        if tk is None or not os.path.exists(tcl):
-            self.sysexit("Could not find Tk. Aborting standalone generation.")
+        if tk is None or not os.path.exists(tk):
+            self.sysexit(
+                "Could not find Tk, you might need to set 'TK_LIBRARY' and if that works, report a bug."
+            )
 
-        # survived the above, now do the copying to following locations
-        target_tk = os.path.join(dist_dir, "tk")
-        target_tcl = os.path.join(dist_dir, "tcl")
+        # survived the above, now do provide the locations
+        yield makeIncludedDataDirectory(
+            source_path=tk,
+            dest_path="tk",
+            reason="Tk copy needed for standalone Tcl",
+            ignore_dirs=("demos",),
+        )
+        yield makeIncludedDataDirectory(
+            source_path=tcl,
+            dest_path="tcl",
+            reason="Tcl needed for tkinter usage",
+        )
 
-        copyTree(tk, target_tk)
-        self.info("Copied Tk libraries from %s." % tk)  # just to entertain
-
-        # Definitely don't need the demos, so remove them again.
-        # TODO: Anything else?
-        shutil.rmtree(os.path.join(target_tk, "demos"), ignore_errors=True)
-
-        copyTree(tcl, target_tcl)
-        self.info("Copied TCL libraries from %s." % tcl)  # just to entertain
-
-        return ()
+        if isWin32Windows():
+            yield makeIncludedDataDirectory(
+                source_path=os.path.join(tcl, "..", "tcl8"),
+                dest_path="tcl8",
+                reason="Tcl modules needed for tkinter usage",
+            )
 
 
-class TkinterPluginDetector(NuitkaPluginBase):
+class NuitkaPluginDetectorTkinter(NuitkaPluginBase):
     """Used only if plugin is not activated.
 
     Notes:
         We are given the chance to issue a warning if we think we may be required.
     """
 
-    detector_for = TkinterPlugin
+    detector_for = NuitkaPluginTkinter
 
     @classmethod
     def isRelevant(cls):
